import type * as postcss from 'postcss';
import { type ImmutableSelectorNode } from '@tokey/css-selector-parser';
import * as CSSValue from '@tokey/css-value-parser';
import type { Invalid } from './invalid-node';
import { ParseForEditingResult } from './edit-time-parser';
export type NodeType = postcss.AnyNode | ImmutableSelectorNode | CSSValue.BaseAstNode;
export type AstLocation = {
    type: 'base';
    node: postcss.AnyNode;
    offsetInNode: number;
    where: 'root' | 'comment' | 'ruleSelector' | 'ruleBetweenSelectorAndBody' | 'ruleBody' | 'atRuleName' | 'atRuleParams' | 'atRuleBody' | 'declProp' | 'declBetweenPropAndColon' | 'declValue' | 'invalid';
} | {
    type: 'selector';
    node: ImmutableSelectorNode;
    offsetInNode: number;
    parents: NodeType[];
    afterSelector: boolean;
} | {
    type: 'declValue';
    ast: CSSValue.BaseAstNode[];
    node: CSSValue.BaseAstNode | CSSValue.BaseAstNode[];
    offsetInNode: number;
    parents: NodeType[];
} | {
    type: 'atRuleParams';
    ast: CSSValue.BaseAstNode[];
    node: CSSValue.BaseAstNode | CSSValue.BaseAstNode[];
    offsetInNode: number;
    parents: NodeType[];
};
export interface AstLocationResult {
    base: AstLocation & {
        type: 'base';
    };
    selector: (AstLocation & {
        type: 'selector';
    }) | undefined;
    declValue: (AstLocation & {
        type: 'declValue';
    }) | undefined;
    atRuleParams: (AstLocation & {
        type: 'atRuleParams';
    }) | undefined;
}
export declare function getAstNodeAt(parseData: ParseForEditingResult, targetOffset: number): AstLocationResult;
export declare function isRule(node: any): node is postcss.Rule;
export declare function isDeclaration(node: any): node is postcss.Declaration;
export declare function isAtRule(node: any): node is postcss.AtRule;
export declare function isInvalid(node: any): node is Invalid;
//# sourceMappingURL=ast-from-position.d.ts.map