"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AMBIGUITY = exports.ERRORS = exports.parseForEditing = void 0;
const stringifier_1 = require("./stringifier");
const invalid_node_1 = require("./invalid-node");
const tokenize_1 = __importDefault(require("postcss/lib/tokenize"));
const parser_1 = __importDefault(require("postcss/lib/parser"));
const postcss = __importStar(require("postcss"));
function parseForEditing(source, { from = '' } = {}) {
    const input = new postcss.Input(source, { from }); // ToDo: check why stringifier option doesn't work
    const parser = new EditTimeParser(input);
    parser.parse();
    parser.closeAstSource();
    return {
        ast: parser.root,
        errorNodes: parser.errorNodes,
        ambiguousNodes: parser.ambiguousNodes,
    };
}
exports.parseForEditing = parseForEditing;
exports.ERRORS = {
    RULE_MISSING_OPEN: 'missing rule open {',
    MISSING_CLOSE: 'missing close }',
    UNCLOSED_BRACKETS: 'found unclosed brackets',
    DECL_MISSING_COLON: 'missing declaration colon',
    UNEXPECTED_CLOSE: 'unexpected close',
    ATRULE_MISSING_NAME: 'missing at-rule name',
};
exports.AMBIGUITY = {
    POSSIBLE_UNOPENED_RULE: 'potential up-opened rule',
};
class EditTimeParser extends parser_1.default {
    constructor(input) {
        super(input);
        this.errorNodes = new Map();
        this.ambiguousNodes = new Map();
        (0, stringifier_1.overrideNodeStringifier)(this.current);
    }
    createTokenizer() {
        this.tokenizer = (0, tokenize_1.default)(this.input, { ignoreErrors: true });
    }
    closeAstSource(node = this.root) {
        if (!node.source.end) {
            const nodes = node.nodes;
            // ToDo: add before/between/after values
            if (!nodes || nodes.length === 0) {
                const startPos = node.source.start;
                node.source.end = {
                    offset: startPos.offset + node.source.input.css.length,
                    line: startPos.line,
                    column: startPos.column,
                };
            }
            else {
                const lastNode = nodes[nodes.length - 1];
                this.closeAstSource(lastNode);
                const closePos = lastNode.source.end;
                node.source.end = {
                    offset: closePos.offset + (node.raws.after?.length || 0),
                    line: closePos.line,
                    column: closePos.column,
                };
            }
        }
    }
    reportNode(node, type, report) {
        const bucket = type === 'error' ? this.errorNodes : this.ambiguousNodes;
        if (!bucket.has(node)) {
            bucket.set(node, []);
        }
        bucket.get(node).push(report);
    }
    createInvalidNode(tokens, assumeTypes) {
        const node = new invalid_node_1.Invalid();
        assumeTypes.forEach((type) => node.assume.add(type));
        this.init(node, tokens[0][2]);
        this.raw(node, 'value', tokens, /* keep spaces*/ true);
        node.source.end = this.getPosition(this.getLastOffset(tokens));
        node.source.end.offset++;
        this.extendParentEnd(node);
        return node;
    }
    getLastOffset(tokens) {
        let collectedOffset = 0;
        for (let i = tokens.length - 1; i >= 0; --i) {
            const current = tokens[i];
            const offset = current[3] || current[2];
            if (offset !== undefined) {
                return offset + collectedOffset;
            }
            else {
                collectedOffset += current[1].length;
            }
        }
        return -1;
    }
    extendParentEnd(node) {
        if (node.parent) {
            node.parent.source.end = node.source.end;
        }
    }
    unOpenedRule(tokens) {
        const node = this.createInvalidNode(tokens, ['rule']);
        this.reportNode(node, 'error', exports.ERRORS.RULE_MISSING_OPEN);
        return node;
    }
    unOpenedRuleOrDecl(tokens, hasColon) {
        const node = this.createInvalidNode(tokens, ['rule', 'decl']);
        this.reportNode(node, 'error', exports.ERRORS.RULE_MISSING_OPEN);
        if (!hasColon) {
            this.reportNode(node, 'error', exports.ERRORS.DECL_MISSING_COLON);
        }
        return node;
    }
    /* --- mods --- */
    init(node, offset) {
        // override with custom stringifier
        (0, stringifier_1.overrideNodeStringifier)(node);
        super.init(node, offset);
    }
    // unclosedBracket - override call in other()
    unknownWord() {
        // 1. overridden call in other()
        // 2. shouldn't happen: from decl with no prop
        // 3..... ToDo: describe cases
    }
    unexpectedClose(token) {
        // keep unexpected in ast
        this.current.raws.after += token[1];
        this.reportNode(this.current, 'error', exports.ERRORS.UNEXPECTED_CLOSE);
    }
    unclosedBlock() {
        // called on unexpected non top node on end-of-file
        let unclosedNode = this.current;
        while (unclosedNode.parent) {
            this.extendParentEnd(unclosedNode);
            unclosedNode = unclosedNode.parent;
        }
        this.reportNode(this.current, 'error', exports.ERRORS.MISSING_CLOSE);
    }
    doubleColon() {
        // no error reported
    }
    unnamedAtrule(node) {
        this.reportNode(node, 'error', exports.ERRORS.ATRULE_MISSING_NAME);
    }
    checkMissedSemicolon() {
        // ignore colon in non custom property decl value
        // no error reported
    }
    other(start) {
        let end = false;
        let type = null;
        let wordBeforeColon = false;
        let colonIndex = -1;
        let colon = false;
        let bracket = null;
        const brackets = [];
        const customProperty = start[1].startsWith('--');
        const tokens = [];
        let token = start;
        while (token) {
            type = token[0];
            tokens.push(token);
            if (type === '(' || type === '[') {
                if (!bracket)
                    bracket = token;
                brackets.push(type === '(' ? ')' : ']');
            }
            else if (customProperty && colon && type === '{') {
                if (!bracket)
                    bracket = token;
                brackets.push('}');
            }
            else if (brackets.length === 0) {
                if (type === ';') {
                    if (colon) {
                        /* mod: add empty prop if missing */
                        if (!wordBeforeColon) {
                            // must be a number because of "wordBeforeColon"
                            const colonToken = tokens[colonIndex];
                            tokens.splice(colonIndex, 0, [
                                'word',
                                '',
                                colonToken[2],
                                colonToken[2],
                            ]);
                        }
                        this.decl(tokens, customProperty);
                        return;
                    }
                    else {
                        break;
                    }
                }
                else if (type === '{') {
                    this.rule(tokens);
                    return;
                }
                else if (type === '}') {
                    this.tokenizer.back(tokens.pop());
                    end = true;
                    break;
                }
                else if (type === ':') {
                    colon = true;
                    colonIndex = tokens.length - 1;
                }
            }
            else if (type === brackets[brackets.length - 1]) {
                brackets.pop();
                if (brackets.length === 0)
                    bracket = null;
            }
            else if (type === 'word') {
                if (!colon) {
                    wordBeforeColon = true;
                }
            }
            token = this.tokenizer.nextToken();
        }
        if (this.tokenizer.endOfFile())
            end = true;
        let node;
        if (this.current.type === 'root') {
            // assume rules only in top level
            node = this.unOpenedRule(tokens);
        }
        else if (end && colon) {
            // declaration-like last node
            if (!customProperty) {
                while (tokens.length) {
                    const tokenType = tokens[tokens.length - 1][0];
                    if (tokenType !== 'space' && tokenType !== 'comment')
                        break;
                    this.tokenizer.back(tokens.pop());
                }
            }
            if (!wordBeforeColon) {
                const colonToken = tokens[colonIndex];
                tokens.splice(colonIndex, 0, ['word', '', colonToken[2], colonToken[2]]);
            }
            this.decl(tokens, customProperty);
            node = this.current.nodes[this.current.nodes.length - 1];
            this.reportNode(node, 'ambiguity', exports.AMBIGUITY.POSSIBLE_UNOPENED_RULE);
        }
        else {
            // ambiguity: might be a rule or a declaration
            node = this.unOpenedRuleOrDecl(tokens, colon);
        }
        // keep track of  unclosed brackets
        if (brackets.length > 0) {
            this.reportNode(node, 'error', exports.ERRORS.UNCLOSED_BRACKETS);
        }
    }
}
//# sourceMappingURL=edit-time-parser.js.map