"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getCompletions = void 0;
const index_internal_1 = require("@stylable/core/dist/index-internal");
const path_1 = __importDefault(require("path"));
const completion_types_1 = require("../../lib/completion-types");
function getCompletions(context) {
    const completions = [];
    const selectorContext = context.getSelectorContext();
    if (!selectorContext) {
        return completions;
    }
    const { resolvedSelectorChain = [], nodeAtCursor = null, fullSelectorAtCursor = '', selectorAtCursor = '', inferredSelectorContext, } = selectorContext;
    if (selectorAtCursor === '::') {
        // No state completion directly after pseudo_element opener: bailout
        return completions;
    }
    const current = resolvedSelectorChain[resolvedSelectorChain.length - 1] || {
        inferred: inferredSelectorContext,
    };
    if (current?.inferred) {
        const pos = context.getPosition();
        const states = current.inferred.getPseudoClasses();
        const isInStateParen = fullSelectorAtCursor.match(/:(.+)\((\w*)$/);
        const inStateParenDef = isInStateParen && states[isInStateParen?.[1]];
        const enumParamDef = getEnumParamDef(inStateParenDef?.state);
        if (enumParamDef) {
            // complete enum options
            const currentParam = isInStateParen[2];
            const completeFullSelector = currentParam === '';
            for (const option of enumParamDef.arguments) {
                if (typeof option !== 'string') {
                    continue;
                }
                let startDelta = 0;
                if (option.startsWith(currentParam)) {
                    // partial completion
                    startDelta = currentParam.length;
                }
                else if (!completeFullSelector) {
                    continue;
                }
                completions.push((0, completion_types_1.stateEnumCompletion)(option, 
                // eslint-disable-next-line @typescript-eslint/no-unnecessary-type-assertion
                normalizeDefinitionSheetLocation(context.meta, inStateParenDef.meta), (0, completion_types_1.range)(pos, { deltaStart: -startDelta })));
            }
        }
        else {
            // complete custom states
            const completeFullSelector = suggestCompleteStates(nodeAtCursor, selectorAtCursor, states, context.meta);
            for (const [name, { state, meta }] of Object.entries(states)) {
                const isAlreadyUsed = current['pseudo_class']?.find((exist) => exist.value === name);
                if (isAlreadyUsed) {
                    continue;
                }
                const stateSelector = ':' + name;
                let startDelta = 0;
                if (stateSelector.startsWith(selectorAtCursor)) {
                    // partial completion
                    startDelta = selectorAtCursor.length;
                }
                else if (!completeFullSelector) {
                    continue;
                }
                const stateWithParam = !!(state && typeof state === 'object');
                const stateType = stateWithParam ? state.type : null;
                const originFile = normalizeDefinitionSheetLocation(context.meta, meta);
                completions.push((0, completion_types_1.stateCompletion)(name, originFile, (0, completion_types_1.range)(pos, { deltaStart: -startDelta }), stateType, stateWithParam));
            }
        }
    }
    return completions;
}
exports.getCompletions = getCompletions;
function getEnumParamDef(stateDef) {
    if (!stateDef || typeof stateDef !== 'object') {
        return;
    }
    else if (stateDef.type === 'enum') {
        return stateDef;
    }
    else if (index_internal_1.STCustomState.isTemplateState(stateDef) && stateDef.params[0].type === 'enum') {
        return stateDef.params[0];
    }
    return;
}
function suggestCompleteStates(nodeAtCursor, selectorAtCursor, states, contextMeta) {
    return (
    // prev is not a pseudo class
    nodeAtCursor?.type !== 'pseudo_class' ||
        // prev is a known native pseudo class
        index_internal_1.nativePseudoClasses.includes(selectorAtCursor) ||
        // prev is a known custom state
        !!states[selectorAtCursor.slice(1)] ||
        // prev is a known custom selector
        !!index_internal_1.STCustomSelector.getCustomSelector(contextMeta, selectorAtCursor.slice(3)));
}
function normalizeDefinitionSheetLocation(originMeta, defMeta) {
    if (defMeta === originMeta) {
        return 'Local file';
    }
    let relPath = path_1.default.relative(path_1.default.dirname(originMeta.source), defMeta.source);
    if (!relPath.startsWith('.')) {
        relPath = './' + relPath;
    }
    return relPath;
}
//# sourceMappingURL=ls-css-pseudo-class.js.map