import { TextDocument } from 'vscode-css-languageservice';
import { Stylable, StylableMeta } from '@stylable/core';
import type { StylableFile } from '../lib/service';
import { getAstNodeAt } from './ast-from-position';
import { ParseReport } from './edit-time-parser';
import { InferredSelector } from '@stylable/core/dist/index-internal';
import { ImmutableSelectorNode } from '@tokey/css-selector-parser';
import type { IFileSystem } from '@file-services/types';
type ResolveChainItem = {
    [Type in ImmutableSelectorNode['type']]?: Extract<ImmutableSelectorNode, {
        type: Type;
    }>[];
} & {
    inferred?: InferredSelector;
};
export declare class LangServiceContext {
    fs: IFileSystem;
    stylable: Stylable;
    private fileData;
    private offset;
    meta: StylableMeta;
    errorNodes: Map<any, ParseReport[]>;
    ambiguousNodes: Map<any, ParseReport[]>;
    location: ReturnType<typeof getAstNodeAt>;
    document: TextDocument;
    flags: {
        runNativeCSSService: boolean;
    };
    constructor(fs: IFileSystem, stylable: Stylable, fileData: StylableFile, offset: number);
    getPosition(offset?: number): import("vscode-languageserver-textdocument").Position;
    isInRoot(): boolean;
    isInSelectorAllowedSpace(): boolean;
    isInSelector(): boolean;
    isInPotentialEmptySelector(): boolean;
    getSelectorContext(): {
        inferredSelectorContext: InferredSelector;
        nodeAtCursor: ImmutableSelectorNode | undefined;
        selectorAtCursor: string;
        fullSelectorAtCursor: string;
        selectorAstResolveMap: Map<ImmutableSelectorNode, InferredSelector>;
        nestedSelectors: (readonly import("@tokey/css-selector-parser/dist/types").ImmutableMap<import("@tokey/css-selector-parser").Selector>[])[];
        resolvedSelectorChain: ResolveChainItem[];
    } | null;
    /**
     * Find the selector that the caret is at and traverse back to collect
     * resolved elements with any selector information after them.
     */
    private aggregateResolvedChain;
    private getSelectorString;
    private collectSelector;
}
export {};
//# sourceMappingURL=lang-service-context.d.ts.map