"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LangServiceContext = void 0;
const vscode_css_languageservice_1 = require("vscode-css-languageservice");
const core_1 = require("@stylable/core");
const ast_from_position_1 = require("./ast-from-position");
const edit_time_parser_1 = require("./edit-time-parser");
const index_internal_1 = require("@stylable/core/dist/index-internal");
const vscode_uri_1 = require("vscode-uri");
const css_selector_parser_1 = require("@tokey/css-selector-parser");
const postcss = __importStar(require("postcss"));
class LangServiceContext {
    constructor(fs, stylable, fileData, offset) {
        this.fs = fs;
        this.stylable = stylable;
        this.fileData = fileData;
        this.offset = offset;
        this.flags = {
            runNativeCSSService: true,
        };
        const parseResult = (0, edit_time_parser_1.parseForEditing)(fileData.content, {
            from: fileData.path,
        });
        this.errorNodes = parseResult.errorNodes;
        this.ambiguousNodes = parseResult.ambiguousNodes;
        this.location = (0, ast_from_position_1.getAstNodeAt)(parseResult, offset);
        this.meta = new index_internal_1.StylableProcessor(new core_1.Diagnostics(), this.stylable.resolveNamespace).process(parseResult.ast);
        this.document = vscode_css_languageservice_1.TextDocument.create(vscode_uri_1.URI.file(this.meta.source).toString(), 'stylable', this.fileData.stat.mtime.getTime(), this.fileData.content);
    }
    getPosition(offset = this.offset) {
        return this.document.positionAt(offset);
    }
    isInRoot() {
        return this.location.base.node.type === 'root';
    }
    isInSelectorAllowedSpace() {
        return this.isInSelector() || this.isInPotentialEmptySelector();
    }
    isInSelector() {
        return !!this.location.selector;
    }
    isInPotentialEmptySelector() {
        const where = this.location.base.where;
        return where === 'root' || where === 'ruleBody' || where === 'atRuleBody';
    }
    getSelectorContext() {
        if (!this.isInSelectorAllowedSpace()) {
            return null;
        }
        const nestedSelectors = this.collectSelector();
        // collect the CSS resolve chain for each selector node
        const selectorAstResolveMap = new Map();
        const transformer = this.stylable.createTransformer();
        let inferredSelectorNest = undefined;
        for (const selector of nestedSelectors) {
            // ToDo(tech-debt): provide an internal selector resolve without the transformer
            const selectorStr = (0, css_selector_parser_1.stringifySelectorAst)(selector);
            const selectorContext = transformer.createSelectorContext(this.meta, selector, // doesn't mutate due to `selectorContext.transform = false`
            postcss.rule({ selector: selectorStr }), selectorStr, inferredSelectorNest);
            selectorContext.transform = false;
            selectorContext.selectorAstResolveMap = selectorAstResolveMap;
            transformer.scopeSelectorAst(selectorContext);
            inferredSelectorNest = selectorContext.inferredMultipleSelectors;
        }
        const inferredSelectorContext = new index_internal_1.InferredSelector(transformer, this.stylable.resolver.resolveExtends(this.meta, 'root'));
        // reference interest points
        const locationSelector = this.location.selector;
        const nodeAtCursor = locationSelector?.node;
        const resolvedSelectorChain = this.aggregateResolvedChain(nestedSelectors, selectorAstResolveMap, inferredSelectorContext);
        return {
            inferredSelectorContext,
            nodeAtCursor,
            selectorAtCursor: nodeAtCursor
                ? (0, css_selector_parser_1.stringifySelectorAst)(nodeAtCursor).slice(0, this.location.selector.offsetInNode)
                : '',
            fullSelectorAtCursor: nodeAtCursor
                ? (0, css_selector_parser_1.stringifySelectorAst)(nestedSelectors[nestedSelectors.length - 1]).slice(0, nodeAtCursor.start + locationSelector.offsetInNode)
                : '',
            selectorAstResolveMap,
            nestedSelectors,
            resolvedSelectorChain,
        };
    }
    /**
     * Find the selector that the caret is at and traverse back to collect
     * resolved elements with any selector information after them.
     */
    aggregateResolvedChain(nestedSelectors, selectorAstResolveMap, inferredSelectorContext) {
        const resolvedChain = [];
        let item = {};
        const nestingSelectors = [...nestedSelectors];
        const addToItem = (node) => {
            var _a;
            const inferred = node ? selectorAstResolveMap.get(node) : inferredSelectorContext;
            if (inferred) {
                item.inferred = inferred;
                resolvedChain.unshift(item);
                item = {};
            }
            else if (node?.type) {
                item[_a = node.type] ?? (item[_a] = []);
                item[node.type].push(node);
            }
        };
        const collectBack = (node, parents) => {
            addToItem(node);
            const parent = parents.pop();
            if ('nodes' in parent && parent.nodes) {
                const nodes = parent.nodes;
                let fromIndex = nodes.indexOf(node);
                fromIndex = fromIndex === -1 ? nodes.length : fromIndex;
                for (let i = fromIndex - 1; i >= 0; i--) {
                    addToItem(nodes[i]);
                }
            }
            if (parents.length) {
                collectBack(parent, parents);
            }
            else {
                addToItem();
            }
        };
        // find node at cursor and collect back resolved location
        if (this.location.selector) {
            const caretSelector = nestingSelectors.pop();
            const targetSelectorNode = this.location.selector?.node;
            (0, css_selector_parser_1.walk)(caretSelector, (node, _index, _nodes, parents) => {
                if (node === targetSelectorNode) {
                    collectBack(node, parents);
                    return css_selector_parser_1.walk.stopAll;
                }
                return;
            });
        }
        else {
            // caret is not at selector (probably nested)
            addToItem( /* default to context */);
        }
        return resolvedChain;
    }
    getSelectorString(node = this.location.base.node) {
        if (node.type === 'rule') {
            return node.selector;
        }
        if (node.type === 'atrule' && node.name === 'st-scope') {
            return node.params;
        }
        return null;
    }
    collectSelector() {
        const parents = collectPostcssParents(this.location.base.node);
        const results = parents
            .map((node) => {
            return (0, index_internal_1.parseSelectorWithCache)(this.getSelectorString(node) || '', { clone: true });
        })
            .filter((selectors) => selectors.length !== 0);
        if (this.location.selector) {
            // caret is on an existing selector
            let selectorWithCaret = this.location.selector.parents.find((node) => node.type === 'selector');
            if (!selectorWithCaret && this.location.selector.node.type === 'selector') {
                selectorWithCaret = this.location.selector.node;
            }
            results.push([selectorWithCaret]);
        }
        else {
            // caret is not on a selector: resolve selector from base node
            const selector = this.getSelectorString(this.location.base.node);
            if (selector) {
                results.push((0, index_internal_1.parseSelectorWithCache)(selector, { clone: true }));
            }
            if (this.isInPotentialEmptySelector()) {
                // caret is on an empty selector
                results.push([]);
            }
        }
        return results;
    }
}
exports.LangServiceContext = LangServiceContext;
function collectPostcssParents(node) {
    const parents = [];
    let current = node.parent;
    while (current) {
        parents.unshift(current);
        current = current.parent;
    }
    return parents;
}
//# sourceMappingURL=lang-service-context.js.map