import type * as postcss from 'postcss';
import type { CSSResolve, Stylable, StylableMeta } from '@stylable/core';
import { ResolvedElement } from '@stylable/core/dist/index-internal';
import type { IFileSystem } from '@file-services/types';
import { Completion } from './completion-types';
import type { ExtendedTsLanguageService } from './types';
import type { CursorPosition } from './utils/selector-analyzer';
import type { LangServiceContext } from '../lib-new/lang-service-context';
export interface PluginCompletionOptions {
    context: LangServiceContext;
    meta: StylableMeta;
    fs: IFileSystem;
    stylable: Stylable;
    src: string;
    tsLangService: ExtendedTsLanguageService;
    resolvedElements: ResolvedElement[][];
    resolvedRoot: ResolvedElement;
    parentSelector: postcss.Rule | null;
    astAtCursor: postcss.AnyNode;
    lineChunkAtCursor: string;
    lastSelectoid: string;
    fullLineText: string;
    position: ProviderPosition;
    resolved: CSSResolve[];
    currentSelector: string;
    target: CursorPosition;
    isMediaQuery: boolean;
    fakes: postcss.Rule[];
}
export interface LangServicePlugin {
    analyzeCaretLocation?(context: LangServiceContext): void;
    onCompletion(options: PluginCompletionOptions): Completion[];
}
export declare class ProviderPosition {
    line: number;
    character: number;
    constructor(line: number, character: number);
}
export declare class ProviderRange {
    start: ProviderPosition;
    end: ProviderPosition;
    constructor(start: ProviderPosition, end: ProviderPosition);
}
export declare class ProviderLocation {
    uri: string;
    range: ProviderRange;
    constructor(uri: string, range: ProviderRange);
}
export declare function createRange(startLine: number, startPos: number, endline: number, endPos: number): ProviderRange;
export declare const ImportInternalDirectivesPlugin: LangServicePlugin;
export declare const RulesetInternalDirectivesPlugin: LangServicePlugin & {
    isSimpleSelector: (sel: string) => boolean;
};
export declare const TopLevelDirectivePlugin: LangServicePlugin;
export declare const ValueDirectivePlugin: LangServicePlugin & {
    isInsideValueDirective: (wholeLine: string, pos: number) => boolean;
};
export declare const GlobalCompletionPlugin: LangServicePlugin;
export declare const SelectorCompletionPlugin: LangServicePlugin;
export declare const ExtendCompletionPlugin: LangServicePlugin;
export declare const CssMixinCompletionPlugin: LangServicePlugin;
export declare const CodeMixinCompletionPlugin: LangServicePlugin;
export declare const FormatterCompletionPlugin: LangServicePlugin;
export declare const NamedCompletionPlugin: LangServicePlugin & {
    resolveImport: (importName: string, stylable: Stylable, meta: StylableMeta) => StylableMeta | null;
};
export declare const PseudoElementCompletionPlugin: LangServicePlugin;
export declare const StateTypeCompletionPlugin: LangServicePlugin;
export declare const StateSelectorCompletionPlugin: LangServicePlugin;
export declare const ValueCompletionPlugin: LangServicePlugin;
//# sourceMappingURL=completion-providers.d.ts.map