"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.valueCompletion = exports.stateEnumCompletion = exports.stateCompletion = exports.stateTypeCompletion = exports.pseudoElementCompletion = exports.formatterCompletion = exports.codeMixinCompletion = exports.cssMixinCompletion = exports.stImportNamedCompletion = exports.namedCompletion = exports.extendCompletion = exports.classCompletion = exports.globalCompletion = exports.valueDirective = exports.topLevelDirective = exports.rulesetInternalDirective = exports.importInternalDirective = exports.topLevelDirectives = exports.rulesetDirectives = exports.importDirectives = exports.range = exports.Snippet = exports.Completion = void 0;
const css_selector_parser_1 = require("@tokey/css-selector-parser");
class Completion {
    constructor(label, detail = '', sortText = 'd', insertText = label, range, additionalCompletions = false, triggerSignature = false) {
        this.label = label;
        this.detail = detail;
        this.sortText = sortText;
        this.insertText = insertText;
        this.range = range;
        this.additionalCompletions = additionalCompletions;
        this.triggerSignature = triggerSignature;
    }
}
exports.Completion = Completion;
class Snippet {
    constructor(source) {
        this.source = source;
    }
}
exports.Snippet = Snippet;
function range(pos, { deltaStart = 0, deltaEnd = 0 } = {}) {
    return {
        start: { line: pos.line, character: pos.character + deltaStart },
        end: { line: pos.line, character: pos.character + deltaEnd },
    };
}
exports.range = range;
exports.importDirectives = {
    from: `-st-from`,
    default: `-st-default`,
    named: `-st-named`,
};
exports.rulesetDirectives = {
    extends: `-st-extends`,
    mixin: `-st-mixin`,
    states: `-st-states`,
};
exports.topLevelDirectives = {
    root: '.root',
    namespace: '@st-namespace',
    customSelector: '@custom-selector :--',
    vars: ':vars',
    import: ':import',
    stScope: '@st-scope',
    stImport: '@st-import',
    stGlobalCustomProperty: '@st-global-custom-property',
};
// syntactic
function importInternalDirective(type, rng) {
    switch (exports.importDirectives[type]) {
        case `-st-default`:
            return new Completion(`-st-default:`, 'Default export name', 'a', new Snippet(`-st-default` + ': $1;'), rng);
        case `-st-from`:
            return new Completion(`-st-from:`, 'Path to library', 'a', new Snippet(`-st-from` + ': "$1";'), rng);
        case `-st-named`:
            return new Completion(`-st-named:`, 'Named export name', 'a', new Snippet(`-st-named` + ': $1;'), rng);
    }
}
exports.importInternalDirective = importInternalDirective;
function rulesetInternalDirective(type, rng) {
    switch (exports.rulesetDirectives[type]) {
        case `-st-extends`:
            return new Completion(`-st-extends:`, 'Extend an external component', 'a', new Snippet('-st-extends: $1;'), rng, true);
        case `-st-mixin`:
            return new Completion(`-st-mixin:`, 'Apply mixins on the class', 'a', new Snippet('-st-mixin: $1;'), rng, true);
        case `-st-states`:
            return new Completion(`-st-states:`, 'Define the CSS states available for this class', 'a', new Snippet('-st-states: $1;'), rng);
    }
}
exports.rulesetInternalDirective = rulesetInternalDirective;
function topLevelDirective(type, rng) {
    switch (exports.topLevelDirectives[type]) {
        case exports.topLevelDirectives.import:
            return new Completion(exports.topLevelDirectives.import, 'Import an external library', 'a', new Snippet(':import {\n\t-st-from: "$1";\n}'), rng);
        case exports.topLevelDirectives.namespace:
            return new Completion(exports.topLevelDirectives.namespace, 'Declare a namespace for the file', 'a', new Snippet('@st-namespace "$1";\n'), rng);
        case exports.topLevelDirectives.customSelector:
            return new Completion(exports.topLevelDirectives.customSelector.slice(0, -4), 'Define a custom selector', 'a', exports.topLevelDirectives.customSelector, rng);
        case exports.topLevelDirectives.root:
            return new Completion(exports.topLevelDirectives.root, 'The root class', 'a', undefined, rng);
        case exports.topLevelDirectives.vars:
            return new Completion(exports.topLevelDirectives.vars, 'Declare variables', 'a', new Snippet(':vars {\n\t$1\n}'), rng);
        case exports.topLevelDirectives.stScope:
            return new Completion(exports.topLevelDirectives.stScope, 'Define an @st-scope', 'a', new Snippet('@st-scope $1 {\n\t$2\n}'), rng);
        case exports.topLevelDirectives.stImport:
            return new Completion(exports.topLevelDirectives.stImport, 'Define an @st-import', 'a', new Snippet('@st-import $2 from "$1";'), rng);
        case exports.topLevelDirectives.stGlobalCustomProperty:
            return new Completion(exports.topLevelDirectives.stGlobalCustomProperty, 'Define global custom properties using @st-global-custom-property', 'a', new Snippet('@st-global-custom-property --$1;'), rng);
    }
}
exports.topLevelDirective = topLevelDirective;
function valueDirective(rng) {
    return new Completion('value()', 'Use the value of a variable', 'a', new Snippet(' value($1)'), rng);
}
exports.valueDirective = valueDirective;
function globalCompletion(rng) {
    return new Completion(':global()', 'Target a global selector', 'a', new Snippet(':global($1)'), rng);
}
exports.globalCompletion = globalCompletion;
// semantic
function classCompletion(className, rng, removeDot = false) {
    return new Completion((removeDot ? '' : '.') + className, 'Stylable class or tag', 'a', undefined, rng);
}
exports.classCompletion = classCompletion;
function extendCompletion(symbolName, from, rng) {
    return new Completion(symbolName, 'from: ' + from, 'a', new Snippet(symbolName), rng);
}
exports.extendCompletion = extendCompletion;
function namedCompletion(symbolName, rng, from, value) {
    return new Completion(symbolName, 'from: ' + from + '\n' + 'Value: ' + value, 'a', new Snippet(symbolName), rng);
}
exports.namedCompletion = namedCompletion;
function stImportNamedCompletion({ originSymbol, jsValue, localName, rng, relativePath, }) {
    const detail = stImportNamedCompletion.detail({ relativePath, symbol: originSymbol, jsValue });
    return new Completion(localName, detail, 'a', new Snippet(localName), rng);
}
exports.stImportNamedCompletion = stImportNamedCompletion;
stImportNamedCompletion.detail = ({ relativePath, symbol, jsValue, }) => {
    if (symbol) {
        switch (symbol?._kind) {
            case 'class': {
                const global = symbol['-st-global']
                    ? `=> "${(0, css_selector_parser_1.stringifySelectorAst)(symbol['-st-global'][0])}"`
                    : '';
                return `.${symbol.name} ${global} from '${relativePath}'`;
            }
            case 'cssVar': {
                const global = symbol.global ? 'global ' : '';
                return `.${symbol.name} (${global}custom property) from '${relativePath}'`;
            }
            case 'var':
                return `${symbol.name}: \`${symbol.text || ''}\` (build var) from '${relativePath}'`;
            case 'element':
                return `${symbol.name} (type selector) from '${relativePath}'`;
            case 'keyframes': {
                const global = symbol.global ? 'global ' : '';
                return `${symbol.name} (${global}keyframes) from '${relativePath}'`;
            }
            case 'layer': {
                const global = symbol.global ? 'global ' : '';
                return `${symbol.name} (${global}layer) from '${relativePath}'`;
            }
            case 'container': {
                const global = symbol.global ? 'global ' : '';
                return `${symbol.name} (${global}container) from '${relativePath}'`;
            }
        }
    }
    else if (jsValue) {
        const type = typeof jsValue;
        if (type === 'function') {
            const originName = jsValue.name || '';
            if (originName) {
                return `${originName}() from '${relativePath}'`;
            }
        }
        else if (type === 'string') {
            return `"${jsValue}" from '${relativePath}'`;
        }
        else {
            return `${jsValue.toString()} from '${relativePath}'`;
        }
    }
    return `from: ${relativePath}`;
};
stImportNamedCompletion.typeAssertCall = ({ localName, rng, }) => {
    const detail = stImportNamedCompletion.typeAssertCallDetail(localName);
    const triggerCompletion = true;
    return new Completion(localName + '()', detail, 'a', new Snippet(`${localName}($1)`), rng, triggerCompletion);
};
stImportNamedCompletion.typeAssertCallDetail = (type) => {
    switch (type) {
        case 'keyframes':
            return '@keyframes definitions';
        case 'layer':
            return '@layer definitions';
        case 'container':
            return 'container query name definitions';
    }
    return '';
};
function cssMixinCompletion(symbolName, rng, from) {
    return new Completion(symbolName, 'from: ' + from, 'a', new Snippet(symbolName), rng);
}
exports.cssMixinCompletion = cssMixinCompletion;
function codeMixinCompletion(symbolName, rng, from) {
    return new Completion(symbolName, 'from: ' + from, 'a', new Snippet(symbolName + '($1)'), rng, false, true);
}
exports.codeMixinCompletion = codeMixinCompletion;
function formatterCompletion(symbolName, rng, from) {
    return new Completion(symbolName, 'from: ' + from, 'a', new Snippet(symbolName + '($1)'), rng, false, true);
}
exports.formatterCompletion = formatterCompletion;
function pseudoElementCompletion(elementName, from, rng) {
    return new Completion('::' + elementName, 'from: ' + from, 'a', '::' + elementName, rng);
}
exports.pseudoElementCompletion = pseudoElementCompletion;
function stateTypeCompletion(type, from, rng) {
    return new Completion(`${type}()`, `from: ${from}`, 'a', new Snippet(`${type}($1)`), rng, false);
}
exports.stateTypeCompletion = stateTypeCompletion;
function stateCompletion(stateName, from, rng, type, hasParam) {
    return new Completion(':' + stateName + (hasParam ? '()' : ''), 'from: ' + from, 'a', new Snippet(':' + stateName + (hasParam ? '($1)' : '')), rng, type === 'enum', hasParam);
}
exports.stateCompletion = stateCompletion;
function stateEnumCompletion(option, from, rng) {
    return new Completion(option, 'from: ' + from, 'a', new Snippet(option), rng, false);
}
exports.stateEnumCompletion = stateEnumCompletion;
function valueCompletion(name, from, value, rng) {
    return new Completion(name, 'from: ' + from + '\n' + 'value: ' + value, 'a', new Snippet(name), rng);
}
exports.valueCompletion = valueCompletion;
//# sourceMappingURL=completion-types.js.map