import type { IFileSystem, IFileSystemStats } from '@file-services/types';
import type { Stylable } from '@stylable/core';
import type { HoverSettings } from 'vscode-css-languageservice';
import type { ColorPresentationParams } from 'vscode-languageserver-protocol';
import { TextDocument } from 'vscode-languageserver-textdocument';
import { Color, ColorInformation, ColorPresentation, CompletionItem, Diagnostic, Hover, Location, ParameterInformation, Position, SignatureHelp, TextEdit, WorkspaceEdit } from 'vscode-languageserver';
import { ProviderPosition } from './completion-providers';
import { CssService } from './css-service';
import { StylableLangServiceFormattingOptions } from './feature/formatting';
import { Provider } from './provider';
import type { ExtendedTsLanguageService } from './types';
import { LangServiceContext } from '../lib-new/lang-service-context';
export interface StylableLanguageServiceOptions {
    fs: IFileSystem;
    stylable: Stylable;
}
export declare class StylableLanguageService {
    cssService: CssService;
    protected fs: IFileSystem;
    protected provider: Provider;
    protected stylable: Stylable;
    protected tsLanguageService: ExtendedTsLanguageService;
    constructor({ fs, stylable }: StylableLanguageServiceOptions);
    getStylable(): Stylable;
    getFs(): IFileSystem;
    onCompletion(filePath: string, offset: number): CompletionItem[];
    onDefinition(filePath: string, offset: number): Location[];
    onHover(filePath: string, offset: number, settings?: HoverSettings): Hover | null;
    onReferences(filePath: string, offset: number): Location[];
    onDocumentColor(filePath: string): ColorInformation[];
    onColorPresentation(filePath: string, offset: {
        start: number;
        end: number;
    }, color: Color): ColorPresentation[];
    onRenameRequest(filePath: string, offset: number, newName: string): WorkspaceEdit;
    onSignatureHelp(filePath: string, offset: number): SignatureHelp | null;
    onDocumentFormatting(filePath: string, options: StylableLangServiceFormattingOptions): TextEdit[];
    onDocumentRangeFormatting(filePath: string, offset: {
        start: number;
        end: number;
    }, options: StylableLangServiceFormattingOptions): TextEdit[];
    getDocumentFormatting(doc: TextDocument, offset: {
        start: number;
        end: number;
    }, options: StylableLangServiceFormattingOptions): TextEdit[];
    provideCompletionItemsFromSrc(context: LangServiceContext): import("./completion-types").Completion[];
    getCompletions(context: LangServiceContext): CompletionItem[];
    getDefinitionLocation(src: string, position: ProviderPosition, filePath: string): Location[];
    getSignatureHelp(src: string, pos: Position, filePath: string, paramInfo: typeof ParameterInformation): SignatureHelp | null;
    getRefs(filePath: string, position: ProviderPosition): Location[];
    resolveDocumentColors(document: TextDocument): import("vscode-languageserver-types").ColorInformation[];
    getColorPresentation(document: TextDocument, params: ColorPresentationParams): import("vscode-languageserver-types").ColorPresentation[];
    diagnose(filePath: string): Diagnostic[];
    private readStylableFile;
}
export interface StylableFile {
    path: string;
    stat: IFileSystemStats;
    content: string;
}
//# sourceMappingURL=service.d.ts.map