"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.wrapAndCatchErrors = void 0;
function wrapAndCatchErrors(config, context) {
    const proto = context.prototype;
    for (const [name, defaultReturn] of Object.entries(config)) {
        const func = proto[name];
        if (typeof func !== 'function') {
            console.error(`expected to find a function named ${name} on context, but found ${typeof func}`);
            continue;
        }
        proto[name] = function (...args) {
            try {
                return func.apply(this, args) || defaultReturn();
            }
            catch (e) {
                const errorContent = e instanceof Error ? e.stack : e;
                console.error(`\nUnexpected error in ${name}\n`);
                console.log(`${errorContent}\n\n`);
                return;
            }
        };
    }
}
exports.wrapAndCatchErrors = wrapAndCatchErrors;
//# sourceMappingURL=wrap-and-log.js.map